/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.basic;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;

import org.audiochain.ui.gui.GenericAudioDeviceComponent;
import org.audiochain.ui.gui.LightEmittingDiodeCheckBox;

public class BasicAudioDeviceComponent extends GenericAudioDeviceComponent {
	private static final long serialVersionUID = 1L;
	private final BasicAudioDevice audioDevice;

	public BasicAudioDeviceComponent(BasicAudioDevice audioDevice) {
		super(audioDevice);
		this.audioDevice = audioDevice;
		init();
	}

	private void init() {
		// Solo should not be displayed, if this device is on the master track
		if (audioDevice.findAssociatedAudioTrack() != null) {
			final JCheckBox check = new LightEmittingDiodeCheckBox("Solo", audioDevice.isSolo(), 8, 8);
			check.setAlignmentX(CENTER_ALIGNMENT);

			audioDevice.addBasicAudioDeviceListener(new BasicAudioDeviceListener() {
				@Override
				public void soloChanged(boolean oldSolo, boolean newSolo) {
					check.setSelected(newSolo);
				}
			});

			check.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					audioDevice.setSolo(check.isSelected());
				}
			});
			add(check, 1);
		}
	}
}
