/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.basic;

import java.awt.Component;

import org.audiochain.model.CommandLineUserInterface;
import org.audiochain.ui.GenericCommandLineUserInterface;
import org.audiochain.ui.gui.GenericUserInterfaceContext;

public class BasicAudioDeviceUserInterfaceContext extends GenericUserInterfaceContext<BasicAudioDevice> {
	private static final long serialVersionUID = 1L;
	
	
	@Override
	public Component createComponent(BasicAudioDevice audioDevice) {
		return new BasicAudioDeviceComponent(audioDevice);
	}
	
	@Override
	protected CommandLineUserInterface createCommandLineUserInterface(final BasicAudioDevice audioDevice) {
		return new GenericCommandLineUserInterface(audioDevice) {
			@Override
			public boolean modifyAudioDevice(String propertyName, String newValue) throws Exception {
				if(super.modifyAudioDevice(propertyName, newValue)) {
					return true;
				}
				if("ToggleMute".equalsIgnoreCase(propertyName)) {
					audioDevice.toggleMute();
					return true;
				}else if("ToggleSolo".equalsIgnoreCase(propertyName)) {
					audioDevice.toggleSolo();
					return true;
				}

				return false;
			}
			
			@Override
			public String getHelpString() {
				StringBuilder sb = new StringBuilder();
				sb.append(super.getHelpString());
				sb.append(format("ToggleMute", "Toggle Mute of the selected track."));
				sb.append(format("ToggleSolo", "Toggle Solo of the selected track."));
				return sb.toString();
			}
			
			@Override
			public String getDisplayString() {
				StringBuilder sb = new StringBuilder();
				sb.append(super.getDisplayString());
				sb.append(audioDevice.isSolo() ? " Solo" : "");
				return sb.toString();
			}
		};
	}
}
