/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.compound;

import org.audiochain.devices.basic.BasicAudioDevice;
import org.audiochain.devices.level.SamplePeakProgrammeMeterAudioDevice;
import org.audiochain.io.AudioDataReader;
import org.audiochain.io.AudioDataReaderChainLink;
import org.audiochain.io.FrameObserver;
import org.audiochain.io.SourceSinkAudioDataReader;
import org.audiochain.model.AudioProject;
import org.audiochain.model.UserInterfaceContext;


public class BasicSppmAudioDevice extends BasicAudioDevice {
	private static final long serialVersionUID = 1L;
	
	SamplePeakProgrammeMeterAudioDevice samplePeakProgrammeMeterAudioDevice;
	
	public BasicSppmAudioDevice() {
		samplePeakProgrammeMeterAudioDevice = new SamplePeakProgrammeMeterAudioDevice();
	}

	@Override
	public AudioDataReaderChainLink createAudioDataReaderChainLink(AudioDataReader source, float frameRate, int sampleSizeInBits, FrameObserver frameObserver) {
		AudioDataReaderChainLink basicReader = super.createAudioDataReaderChainLink(source, frameRate, sampleSizeInBits, frameObserver);
		AudioDataReader levelReader = samplePeakProgrammeMeterAudioDevice.createAudioDataReaderChainLink(basicReader, frameRate, sampleSizeInBits, frameObserver);
		return new SourceSinkAudioDataReader(basicReader, levelReader);
	}

	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		samplePeakProgrammeMeterAudioDevice.setEnabled(enabled);
	}

	@Override
	public void initialize(AudioProject project) {
		super.initialize(project);
		samplePeakProgrammeMeterAudioDevice.initialize(project);
	}

	@Override
	public UserInterfaceContext<BasicAudioDevice> createUserInterfaceContext() {
		return new BasicSppmUserInterfaceContext();
	}
	
	@Override
	public String getName() {
		return "Volume/SPPM";
	}

	public SamplePeakProgrammeMeterAudioDevice getSamplePeakProgrammeMeterAudioDevice() {
		return samplePeakProgrammeMeterAudioDevice;
	}
}
