/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.drum;

import java.io.File;

import org.audiochain.ui.GenericCommandLineUserInterface;

public class DrumCommandLineUserInterface extends GenericCommandLineUserInterface {
	private DrumAudioDevice device;

	public DrumCommandLineUserInterface(DrumAudioDevice device) {
		super(device);
		this.device = device;
	}

	@Override
	public String getHelpString() {
		StringBuilder sb = new StringBuilder();
		sb.append(format("DrumSchema", "Sets the drum schema file."));
		sb.append(format("Rhythm", "Activates one of the rhythms of the drum schema."));
		sb.append(super.getHelpString());
		return sb.toString();
	}

	@Override
	public String getDisplayString() {
		StringBuilder sb = new StringBuilder();
		sb.append(device.getDrumSchemaFileName()).append(' ');
		sb.append(device.getDrumRhythmName()).append(' ');
		sb.append(super.getDisplayString());
		return sb.toString();
	}

	@Override
	public boolean modifyAudioDevice(String propertyName, String newValue) throws Exception {
		if ("DrumSchema".equals(propertyName)) {
			File schemaFile = device.getAudioProject().getRelativeFile(newValue);
			device.setDrumSchemaFile(schemaFile);
			return true;
		}
		if ("Rhythm".equals(propertyName)) {
			device.setDrumRhythm(newValue);
			return true;
		}
		return super.modifyAudioDevice(propertyName, newValue);
	}
}
