/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.drum;

import java.util.ArrayList;
import java.util.List;

/**
 * | b bc b c s c . c |
 * 
 * @author Florian Paul
 * 
 */
public class DrumPattern {
	private final List<DrumBeat> beats = new ArrayList<DrumBeat>();

	public List<DrumBeat> getBeats() {
		return beats;
	}
	
	void validate() throws DrumSchemaFormatException {
		if(beats.isEmpty()) {
			throw new DrumSchemaFormatException("The drum pattern must have beats");
		}
		int num = 0;
		for(DrumBeat beat : beats) {
			try {
				num++;
				beat.validate();
			} catch (DrumSchemaFormatException ex) {
				throw new DrumSchemaFormatException(ex.getMessage() + " at beat " + num);
			}
		}
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append(' ');
		for (DrumBeat beat : beats) {
			sb.append(beat.toString());
			sb.append(' ');
		}
		return sb.toString();
	}
}
