/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.drum;

import java.util.ArrayList;
import java.util.Collection;

/**
 * 01 | b c . c s c . c | b bc b c s c . c |
 * 
 * @author Florian Paul
 *
 */
public class DrumSequence {
	private final String name;
	private final Collection<DrumPattern> patterns = new ArrayList<DrumPattern>();
	
	public DrumSequence(String name) {
		this.name = name;
	}

	public Collection<DrumPattern> getPatterns() {
		return patterns;
	}
	
	public Collection<DrumBeat> getAllBeats() {
		Collection<DrumBeat> ret = new ArrayList<DrumBeat>();
		for(DrumPattern pattern : patterns) {
			ret.addAll(pattern.getBeats());
		}
		return ret;
	}

	public String getName() {
		return name;
	}
	
	void validate() throws DrumSchemaFormatException {
		if (name == null) {
			throw new DrumSchemaFormatException("The drum sequence must have an identifier (i.e. '01' for a sequence definition like 01 | b c . c s c . c | b bc b c s c . c |)");
		}
		if (patterns.isEmpty()) {
			throw new DrumSchemaFormatException("The drum sequence '" + name + "' has no patterns");
		}
		for (DrumPattern pattern : patterns) {
			try {
				pattern.validate();
			} catch (DrumSchemaFormatException ex) {
				throw new DrumSchemaFormatException(ex.getMessage() + " at sequence '" + name + "'");
			}
		}
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder(name);
		sb.append(' ');
		sb.append('|');
		for(DrumPattern pattern : patterns) {
			sb.append(pattern.toString());
			sb.append('|');
		}
		return sb.toString();
	}
}
