/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.drum;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.Collection;
import java.util.LinkedHashSet;

import org.audiochain.model.ObjectStore;

public class DrumSettings implements Serializable {
	private static final long serialVersionUID = 1L;
	private static final String PROPS_FILE = "drum.properties";
	private static DrumSettings instance;
	private Collection<String> clipSearchPaths;

	DrumSettings() {

	}

	public static synchronized DrumSettings getInstance() {
		if (instance == null) {
			try {
				instance = (DrumSettings) ObjectStore.read(PROPS_FILE);
			} catch (Exception e) {
				// can be ignored, new instance with default values will be used
				e.printStackTrace();
			}
			if (instance == null) {
				instance = new DrumSettings();
				if ("Linux".equalsIgnoreCase(System.getProperty("os.name"))) {
					// add for convenience on Debian Systems
					File hydrogenDrumkits = new File("/usr/share/hydrogen/data/drumkits");
					if (hydrogenDrumkits.exists() && hydrogenDrumkits.canRead()) {
						instance.addClipSearchPath(hydrogenDrumkits);
					}
				}
			}
		}
		return instance;
	}

	private void store() {
		try {
			ObjectStore.storeInSettingsDirectory(this, PROPS_FILE);
		} catch (IOException e) {
			throw new IllegalStateException(e);
		}
	}

	public Collection<File> getClipSearchPaths() {
		Collection<File> ret = new LinkedHashSet<File>();
		if (clipSearchPaths != null) {
			for (String path : clipSearchPaths) {
				ret.add(new File(path));
			}
		}
		return ret;
	}

	public void addClipSearchPath(File clipSearchPath) {
		if (clipSearchPaths == null) {
			clipSearchPaths = new LinkedHashSet<String>();
		}
		clipSearchPaths.add(clipSearchPath.getAbsolutePath());
		store();
	}

	public void removeClipSearchPath(File clipSearchPath) {
		if (clipSearchPaths == null) {
			return;
		}
		clipSearchPaths.remove(clipSearchPath.getAbsolutePath());
		if (clipSearchPaths.isEmpty()) {
			clipSearchPaths = null;
		}
		store();
	}
}
