/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.dynamics;

import org.audiochain.devices.dynamics.EnvelopeAudioDataReader.EnvelopeDetectionType;
import org.audiochain.io.AudioDataReader;
import org.audiochain.io.AudioDataReaderChainLink;
import org.audiochain.io.FrameObserver;
import org.audiochain.model.AbstractChainableAudioDevice;
import org.audiochain.model.AudioDeviceProperty;
import org.audiochain.model.UserInterfaceContext;
import org.audiochain.ui.gui.GenericUserInterfaceContext;

public class CompressorAudioDevice extends AbstractChainableAudioDevice {
	private static final long serialVersionUID = 1L;

	@AudioDeviceProperty(name = "Threshold", max = 0, min = -40, unit = "dB", symbol = 8625)
	private float threshold = 0F;
	@AudioDeviceProperty(name = "Ratio", max = 20, min = 1, unit = ":1", symbol = 8260)
	private float ratio = 1F;
	@AudioDeviceProperty(name = "MakeUpGain", max = 30, min = 0, unit = "dB", symbol = 8648)
	private float makeUpGain = 0F;
	@AudioDeviceProperty(name = "Attack", max = 250, min = 0, unit = "ms", symbol = 10528)
	private float attack = 20F;
	@AudioDeviceProperty(name = "Hold", max = 5000, min = 0, unit = "ms", symbol = 10565)
	private float hold = 0F;
	@AudioDeviceProperty(name = "Release", max = 2500, min = 0, unit = "ms", symbol = 10526)
	private float release = 500F;
	@AudioDeviceProperty(name = "Linked", symbol = 9903, falseSymbol = 9902)
	private boolean linked = true;
	@AudioDeviceProperty(name = "Envelope", symbol = 9993)
	private EnvelopeDetectionType envelopeDetectionType = EnvelopeDetectionType.RealLookAhead;
	private transient CompressorAudioDataReader reader;

	@Override
	public AudioDataReaderChainLink createAudioDataReaderChainLink(AudioDataReader source, float frameRate, int sampleSizeInBits, FrameObserver frameObserver) {
		EnvelopeAudioDataReader envelopeReader = new EnvelopeAudioDataReader(source);
		envelopeReader.setEnvelopeDetectionType(envelopeDetectionType);
		reader = new CompressorAudioDataReader(envelopeReader, sampleSizeInBits, frameRate);
		reader.setThreshold(threshold);
		reader.setRatio(ratio);
		reader.setMakeUpGain(makeUpGain);
		reader.setAttack(attack);
		reader.setHold(hold);
		reader.setRelease(release);
		reader.setLinked(linked);
		return reader;
	}

	public void setThreshold(float threshold) {
		this.threshold = threshold;
		if (reader != null) {
			reader.setThreshold(threshold);
		}
	}

	public void setRatio(float ratio) {
		this.ratio = ratio;
		if (reader != null) {
			reader.setRatio(ratio);
		}
	}

	public void setMakeUpGain(float gain) {
		this.makeUpGain = gain;
		if (reader != null) {
			reader.setMakeUpGain(gain);
		}
	}

	public void setAttack(float attack) {
		this.attack = attack;
		if (reader != null) {
			reader.setAttack(attack);
		}
	}

	public void setRelease(float release) {
		this.release = release;
		if (reader != null) {
			reader.setRelease(release);
		}
	}

	public void setHold(float hold) {
		this.hold = hold;
		if (reader != null) {
			reader.setHold(hold);
		}
	}

	public void setEnvelopeDetectionType(EnvelopeDetectionType envelopeDetectionType) {
		this.envelopeDetectionType = envelopeDetectionType;
		if (reader != null) {
			reader.getReader().setEnvelopeDetectionType(envelopeDetectionType);
		}
	}

	public void setLinked(boolean linked) {
		this.linked = linked;
		if (reader != null) {
			reader.setLinked(linked);
		}
	}

	@Override
	public String getName() {
		return "Compressor";
	}

	@Override
	protected UserInterfaceContext<CompressorAudioDevice> createUserInterfaceContext() {
		return new GenericUserInterfaceContext<CompressorAudioDevice>();
	}
}
