/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.frequency;

import static org.audiochain.devices.frequency.DigitalSignalProcessing.*;

import java.util.Collection;

import org.audiochain.model.AudioDeviceProperty;
import org.audiochain.model.AudioDeviceValue;
import org.audiochain.model.FloatAudioDeviceValue;


public class BandPassFilterAudioDevice extends FilterAudioDevice {
	private static final long serialVersionUID = 1L;
	@AudioDeviceProperty(name="Filter Band", symbol=9716, min=1,unit="Hz")
	protected float filterBandWidth = 1000;

	@Override
	protected double[] createFilter() {
		int frameRate = audioProject.getFrameRate();
		return createBandPass(frameRate, cutOffFrequency, filterBandWidth, transitionBandWidth);
	}

	@Override
	public void createAudioDeviceValues(Collection<AudioDeviceValue> values) {
		super.createAudioDeviceValues(values);
		for (AudioDeviceValue audioDeviceValue : values) {
			if(audioDeviceValue.getName().equals("Filter Band")) {
				if(audioProject!=null) {
					((FloatAudioDeviceValue)audioDeviceValue).setMaxValue(audioProject.getNyquistFrequency());
				}
			}
		}
	}

	@Override
	protected String getFilterType() {
		return "Band Pass";
	}
	
	public void setFilterBandWidth(float filterBandWidth) {
		this.filterBandWidth = filterBandWidth;
		updateImpulseResponse();
	}
}
