/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.frequency;

import static org.audiochain.devices.frequency.DigitalSignalProcessing.*;

import java.util.Collection;
import java.util.LinkedHashSet;

import org.audiochain.io.AudioDataReader;
import org.audiochain.io.AudioDataReaderChainLink;
import org.audiochain.io.FrameObserver;
import org.audiochain.model.AbstractChainableAudioDevice;
import org.audiochain.model.UserInterfaceContext;


public class FftAnalyzerAudioDevice extends AbstractChainableAudioDevice {
	private static final long serialVersionUID = 1L;
	private transient FftAudioDataReader reader;
	private transient Collection<FftAudioDataReaderListener> listeners;

	@Override
	public AudioDataReaderChainLink createAudioDataReaderChainLink(AudioDataReader source, float frameRate, int sampleSizeInBits, FrameObserver frameObserver) {
		if (reader != null) {
			reader.removeFftReaderListeners(listeners);
		}
		reader = new FftAudioDataReader(source, frameRate, frameObserver);
		reader.setWindow(true);
		reader.addFftReaderListeners(listeners);
		setFftLength(128);
		return reader;
	}

	@Override
	public String getName() {
		return "FFT Analyzer";
	}

	@Override
	protected UserInterfaceContext<FftAnalyzerAudioDevice> createUserInterfaceContext() {
		return new FftAnalyzerUserInterfaceContext();
	}

	public void setFftLength(int fftLength) {
		fftLength = nextPowerOfTwo(fftLength);
		if (reader != null) {
			reader.setFftLength(fftLength);
		}
	}

	public void addFftReaderListener(FftAudioDataReaderListener listener) {
		if (reader != null) {
			reader.addFftReaderListener(listener);
		}
		if (listeners == null) {
			listeners = new LinkedHashSet<FftAudioDataReaderListener>();
		}
		listeners.add(listener);
	}

	public void removeFftReaderListener(FftAudioDataReaderListener listener) {
		if (reader != null) {
			reader.removeFftReaderListener(listener);
		}
		if (listeners == null) {
			return;
		}
		listeners.remove(listener);
	}
}
