/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.frequency;

import java.util.Collection;

import org.audiochain.io.AudioDataReader;
import org.audiochain.io.AudioDataReaderChainLink;
import org.audiochain.io.FrameObserver;
import org.audiochain.model.AbstractChainableAudioDevice;
import org.audiochain.model.AudioDeviceProperty;
import org.audiochain.model.AudioDeviceValue;
import org.audiochain.model.FloatAudioDeviceValue;
import org.audiochain.model.UserInterfaceContext;
import org.audiochain.ui.gui.GenericUserInterfaceContext;

abstract public class FilterAudioDevice extends AbstractChainableAudioDevice {
	private static final long serialVersionUID = 1L;
	@AudioDeviceProperty(name="CutOff", symbol=9716, min=0,unit="Hz")
	protected float cutOffFrequency = 1000;
	@AudioDeviceProperty(name="Transition Band", symbol=9716, max=2000, min=25,unit="Hz")
	protected float transitionBandWidth = 1000;
	private transient FftConvolutionAudioDataReader reader;

	@Override
	public final AudioDataReaderChainLink createAudioDataReaderChainLink(AudioDataReader source, float frameRate, int sampleSizeInBits, FrameObserver frameObserver) {
		reader = new FftConvolutionAudioDataReader(source, createFilter());
		return reader;
	}

	abstract protected double[] createFilter();

	abstract protected String getFilterType();

	@Override
	protected void createAudioDeviceValues(Collection<AudioDeviceValue> values) {
		for (AudioDeviceValue audioDeviceValue : values) {
			if(audioDeviceValue.getName().equals("CutOff")) {
				audioDeviceValue.changeName(getFilterType() + " CutOff");
				if(audioProject!=null) {
					((FloatAudioDeviceValue)audioDeviceValue).setMaxValue(audioProject.getNyquistFrequency());
				}
			}
		}
	}

	protected void updateImpulseResponse() {
		if (reader != null) {
			reader.setImpulseResponse(createFilter());
		}
	}

	@Override
	protected UserInterfaceContext<FilterAudioDevice> createUserInterfaceContext() {
		return new GenericUserInterfaceContext<FilterAudioDevice>();
	}

	public void setTransitionBandWidth(float transitionBandWidth) {
		this.transitionBandWidth = transitionBandWidth;
		if (reader != null) {
			updateImpulseResponse();
		}
	}

	public void setCutOffFrequency(float cutOffFrequency) {
		this.cutOffFrequency = cutOffFrequency;
		updateImpulseResponse();
	}

	@Override
	public final String getName() {
		return getFilterType() + " Filter";
	}
}
