/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.frequency;

import java.io.IOException;
import java.util.Collection;

import org.audiochain.io.AudioDataReader;
import org.audiochain.io.FrameObserver;
import org.audiochain.io.SignalGeneratorAudioDataReader;
import org.audiochain.io.SignalGeneratorAudioDataReader.SignalType;
import org.audiochain.model.AbstractAudioDevice;
import org.audiochain.model.AudioDeviceProperty;
import org.audiochain.model.AudioDeviceValue;
import org.audiochain.model.FloatAudioDeviceValue;
import org.audiochain.model.Performable;
import org.audiochain.model.SourceAudioDevice;
import org.audiochain.model.UserInterfaceContext;
import org.audiochain.ui.gui.GenericUserInterfaceContext;

public class SignalGeneratorAudioDevice extends AbstractAudioDevice implements SourceAudioDevice {
	private static final long serialVersionUID = 1L;
	@AudioDeviceProperty(name = "Amplitude", max = 1, min = 0, symbol = 9684)
	private float amplitude = 1;
	@AudioDeviceProperty(name = "Frequency", min = 0, symbol = 9684, unit = "Hz")
	private float frequency = 1000;
	@AudioDeviceProperty(name = "Signal Type", symbol = 9684)
	private SignalType signalType = SignalType.Sine;
	private transient SignalGeneratorAudioDataReader reader;

	@Override
	protected void createAudioDeviceValues(Collection<AudioDeviceValue> values) {
		for (AudioDeviceValue value : values) {
			if (value.getName().equals("Frequency")) {
				((FloatAudioDeviceValue) value).setMaxValue(audioProject == null ? 0 : audioProject.getNyquistFrequency());
			}
		}
	}

	@Override
	public String getName() {
		return "Signal Generator";
	}

	@Override
	public AudioDataReader createAudioDataReader(float frameRate, int sampleSizeInBits, FrameObserver frameObserver) throws IOException {
		reader = new SignalGeneratorAudioDataReader(sampleSizeInBits, frameRate);
		reader.setAmplitude(amplitude);
		reader.setFrequency(frequency);
		reader.setSignalType(signalType);
		return reader;
	}

	@Override
	public long getMaxFrame() {
		return -1;
	}

	@Override
	public boolean isAudioDataAvailable() {
		return true;
	}

	@Override
	public boolean isRecordable() {
		return false;
	}

	@Override
	public Performable getPerformable() {
		return null;
	}

	@Override
	protected UserInterfaceContext<SignalGeneratorAudioDevice> createUserInterfaceContext() {
		return new GenericUserInterfaceContext<SignalGeneratorAudioDevice>();
	}

	public void setAmplitude(float amplitude) {
		this.amplitude = amplitude;
		if (reader != null) {
			reader.setAmplitude(amplitude);
		}
	}

	public void setFrequency(float frequency) {
		this.frequency = frequency;
		if (reader != null) {
			reader.setFrequency(frequency);
		}
	}

	public void setSignalType(SignalType signalType) {
		this.signalType = signalType;
		if (reader != null) {
			reader.setSignalType(signalType);
		}
	}
}