/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.latency;

import javax.swing.Box;
import javax.swing.BoxLayout;

import org.audiochain.ui.gui.LabeledRotaryPotentiometerComponent;
import org.audiochain.ui.gui.RotaryPotentiometerImageComponent;
import org.audiochain.ui.gui.RotaryPotentiometerValueChangeListener;

public class LatencyAudioDeviceComponent extends Box {
	private static final long serialVersionUID = 1L;
	private final LatencyAudioDevice device;

	public LatencyAudioDeviceComponent(LatencyAudioDevice device) {
		super(BoxLayout.Y_AXIS);
		this.device = device;
		init();
	}

	private void init() {
		setOpaque(false);

		final LabeledRotaryPotentiometerComponent comp = new LabeledRotaryPotentiometerComponent("Latency", 0F);
		RotaryPotentiometerImageComponent poti = comp.getPotentiometer();
		poti.setMinValue(LatencyAudioDevice.MIN_LATENCY);
		poti.setMaxValue(LatencyAudioDevice.MAX_LATENCY);
		poti.setIntegralValue(true);
		poti.setUnit("frames");
		add(comp);

		comp.setValue(device.getLatency());
		poti.addValueChangeListener(new RotaryPotentiometerValueChangeListener() {
			@Override
			public void rotaryPotentiometerValueChanged(float newValue, float oldValue) {
				device.setLatency((int) newValue);
			}
		});
		device.addLatencyAudioDeviceListener(new LatencyAudioDeviceListener() {
			@Override
			public void latencyChanged(int oldValue, int newValue) {
				comp.setValue(newValue);
			}
		});
	}
}
