/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.level;

import java.text.DecimalFormat;

import org.audiochain.model.AbstractCommandLineUserInterface;
import org.audiochain.model.AudioMixer;
import org.audiochain.ui.PeakProgrammeMeterEvent;
import org.audiochain.ui.PeakProgrammeMeterVisualizer;

public class SamplePeakProgrammeMeterCommandLineUserInterface extends AbstractCommandLineUserInterface implements PeakProgrammeMeterVisualizer {

	private int barsize = 15;
	private String result = "";
	private DecimalFormat df = new DecimalFormat("00.0dB");

	private final SamplePeakProgrammeMeterAudioDevice device;

	SamplePeakProgrammeMeterCommandLineUserInterface(SamplePeakProgrammeMeterAudioDevice device) {
		this.device = device;
		device.addPeakProgrammeMeterVisualizer(this);
	}

	@Override
	public void updatePeakProgrammeMeter(PeakProgrammeMeterEvent event) {
		StringBuilder sb = new StringBuilder();
		sb.append('L');
		makebar(sb, event.getLeftValue(), event.getLeftPeakValue(), event.isLeftClip());
		sb.append(df.format(event.getLeftPeakInDecibel()));

		sb.append(' ');
		sb.append('R');
		makebar(sb, event.getRightValue(), event.getRightPeakValue(), event.isRightClip());
		sb.append(df.format(event.getRightPeakInDecibel()));

		result = sb.toString();
		fireCommandLineUserInterfaceUpdate();
	}

	private void makebar(StringBuilder sb, float value, float peak, boolean clip) {
		int valueTen = (int) (value * barsize);
		int peakTen = (int) (peak * barsize);
		sb.append('[');
		for (int i = 0; i < valueTen; i++) {
			sb.append('#');
		}
		for (int i = valueTen; i < barsize; i++) {
			if (peakTen > 0 && i == peakTen) {
				sb.append('#');
			} else {
				sb.append(' ');
			}
		}
		sb.append(']');
		sb.append(clip ? 'C' : ' ');
	}

	@Override
	public String getDisplayString() {
		return "#";
	}
	
	@Override
	public String getPerformanceDisplayString() {
		return result;
	}

	public int getBarsize() {
		return barsize;
	}

	public void setBarsize(int barsize) {
		this.barsize = barsize;
	}

	@Override
	public String getHelpString() {
		return format("ShowSPPM", "Show the Sample Peak Programme Meter for the recording channel.");
	}

	@Override
	public boolean modifyAudioDevice(String propertyName, String newValue) throws Exception {
		if ("ShowSPPM".equals(propertyName)) {
			PeakProgrammeMeterComponent.showPpm(device.getAudioProject().getAudioFormat(), AudioMixer.getInstance().getCaptureMixer().getMixer());
			return true;
		}
		return false;
	}

}
