/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.reverb;

import org.audiochain.io.AudioDataReader;
import org.audiochain.io.AudioDataReaderChainLink;
import org.audiochain.io.FrameObserver;
import org.audiochain.model.AbstractChainableAudioDevice;
import org.audiochain.model.AudioDeviceProperty;
import org.audiochain.model.UserInterfaceContext;
import org.audiochain.ui.gui.GenericUserInterfaceContext;


public class ReverbAudioDevice extends AbstractChainableAudioDevice {
	private static final long serialVersionUID = 1L;
	@AudioDeviceProperty(name = "Delay L", max = 1500, min = 0, symbol = 9716, unit = "ms")
	private int leftDelayTimeInMilliseconds = 91;
	@AudioDeviceProperty(name = "Delay R", max = 1500, min = 0, symbol = 9716, unit = "ms")
	private int rightDelayTimeInMilliseconds = 102;
	@AudioDeviceProperty(name = "Decay L", max = 1, min = 0, symbol = 10136)
	private float leftDecay = 0.71F;
	@AudioDeviceProperty(name = "Decay R", max = 1, min = 0, symbol = 10136)
	private float rightDecay = 0.75F;
	// max must be equal to ReverbAudioDataReader.DELAY_TIME_MULTIPLIER.length
	@AudioDeviceProperty(name = "Depth", max = 7, min = 1, symbol = 9717)
	private int delayCount = 3;
	@AudioDeviceProperty(name = "Dry Ratio", max = 1, min = 0, symbol = 1016)
	private float dryRatio = 0.5F;
	private transient ReverbAudioDataReader reader;

	@Override
	public AudioDataReaderChainLink createAudioDataReaderChainLink(AudioDataReader source, float frameRate, int sampleSizeInBits, FrameObserver frameObserver) {
		reader = new ReverbAudioDataReader(source, frameRate);
		reader.setLeftDecay(leftDecay);
		reader.setRightDecay(rightDecay);
		reader.setLeftDelayTimeInMilliseconds(leftDelayTimeInMilliseconds);
		reader.setRightDelayTimeInMilliseconds(rightDelayTimeInMilliseconds);
		reader.setAllPassCount(delayCount);
		reader.setDryRatio(dryRatio);
		return reader;
	}

	@Override
	protected UserInterfaceContext<ReverbAudioDevice> createUserInterfaceContext() {
		return new GenericUserInterfaceContext<ReverbAudioDevice>();
	}
	
	public void setLeftDecay(float decay) {
		this.leftDecay = decay;
		if (reader != null) {
			reader.setLeftDecay(decay);
		}
	}

	public void setRightDecay(float decay) {
		this.rightDecay = decay;
		if (reader != null) {
			reader.setRightDecay(decay);
		}
	}

	public void setLeftDelayTimeInMilliseconds(int delayTimeInMilliseconds) {
		this.leftDelayTimeInMilliseconds = delayTimeInMilliseconds;
		if (reader != null) {
			reader.setLeftDelayTimeInMilliseconds(delayTimeInMilliseconds);
		}
	}

	public void setRightDelayTimeInMilliseconds(int delayTimeInMilliseconds) {
		this.rightDelayTimeInMilliseconds = delayTimeInMilliseconds;
		if (reader != null) {
			reader.setRightDelayTimeInMilliseconds(delayTimeInMilliseconds);
		}
	}

	public void setDelayCount(int delayCount) {
		this.delayCount = delayCount;
		if (reader != null) {
			reader.setAllPassCount(delayCount);
		}
	}

	public void setDryRatio(float dryRatio) {
		this.dryRatio = dryRatio;
		if (reader != null) {
			reader.setDryRatio(dryRatio);
		}
	}

	@Override
	public String getName() {
		return "Reverb";
	}
}
