/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.reverb;

public class StereoDelay {
	private final int leftDelayBufferSize;
	private final int rightDelayBufferSize;
	private final int[] leftDelayBuffer;
	private final int[] rightDelayBuffer;
	private int leftBufferIndex;
	private int rightBufferIndex;

	public StereoDelay(float frameRate, int leftDelayTimeInMilliseconds, int rightDelayTimeInMilliseconds) {
		leftDelayBufferSize = (int) (leftDelayTimeInMilliseconds * frameRate / 1000F);
		rightDelayBufferSize = (int) (rightDelayTimeInMilliseconds * frameRate / 1000F);
		leftDelayBuffer = new int[leftDelayBufferSize];
		rightDelayBuffer = new int[rightDelayBufferSize];
	}

	public StereoDelay(int leftDelayBufferSize, int rightDelayBufferSize) {
		this.leftDelayBufferSize = leftDelayBufferSize;
		this.rightDelayBufferSize = rightDelayBufferSize;
		leftDelayBuffer = new int[leftDelayBufferSize];
		rightDelayBuffer = new int[rightDelayBufferSize];
	}

	public void delay(int[] data) {
		for (int l = 0, r = 1, n = data.length; l < n; l += 2, r += 2) {
			if (leftDelayBufferSize > 0) {
				int delayedLeftValue = leftDelayBuffer[leftBufferIndex];
				// remember new value
				leftDelayBuffer[leftBufferIndex] = data[l];
				// return any delayed previous value
				data[l] = delayedLeftValue;
				leftBufferIndex++;
				if (leftBufferIndex >= leftDelayBufferSize) {
					leftBufferIndex = 0;
				}
			}

			if (rightDelayBufferSize > 0) {
				int delayedRightValue = rightDelayBuffer[rightBufferIndex];
				// remember new value
				rightDelayBuffer[rightBufferIndex] = data[r];
				// return any delayed previous value
				data[r] = delayedRightValue;
				rightBufferIndex++;
				if (rightBufferIndex >= rightDelayBufferSize) {
					rightBufferIndex = 0;
				}
			}
		}
	}
}
