/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

import java.io.IOException;
import java.io.InputStream;

/**
 * Provides an InputStream that is reading from an underlying AudioDataReader source.
 * This class makes a conversion of the unified audio data from int array to a sequencial
 * byte array with consideration of the endianess.
 * 
 * Nowhere used for now.
 */
public class AudioDataInputStream extends InputStream {

	private AudioDataReader reader;
	private int audioDataBufferSize;
	private int[] audioDataBuffer;
	private final boolean bigEndian;
	
	public AudioDataInputStream(AudioDataReader reader) {
		this(reader, true);
	}
	
	public AudioDataInputStream(AudioDataReader reader, boolean bigEndian) {
		this.reader = reader;
		this.bigEndian = bigEndian;
	}
	
	@Override
	public int read(byte[] b, int off, int len) throws IOException {
		if(audioDataBuffer==null) {
			audioDataBufferSize = len / 2;
			audioDataBuffer = new int[audioDataBufferSize];
		}
		
		int readAudioByteCount = reader.read(audioDataBuffer);
		if(readAudioByteCount==-1) {
			return -1;
		}
		
		if(bigEndian) {
			for (int i = 0, n = audioDataBuffer.length; i < n; i++) {
				int d = audioDataBuffer[i];
				byte msb = (byte) (255 & (d >> 8));
				byte lsb = (byte) (255 & d);
				b[2 * i] = msb;
				b[2 * i + 1] = lsb;
			}
		} else {
			for (int i = 0, n = audioDataBuffer.length; i < n; i++) {
				int d = audioDataBuffer[i];
				byte msb = (byte) (255 & (d >> 8));
				byte lsb = (byte) (255 & d);
				b[2 * i] = lsb;
				b[2 * i + 1] = msb;
			}
		}
		
		int ret = readAudioByteCount * 2;
		return ret;
	}
	
	@Override
	public int read() {
		throw new IllegalStateException("read not supported");
	}

	public int getBufferSize() {
		return audioDataBufferSize;
	}

	public boolean isBigEndian() {
		return bigEndian;
	}

	@Override
	public void close() throws IOException {
		reader.close();
		audioDataBuffer = null;
	}

}
