/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

import java.io.IOException;

import javax.sound.sampled.AudioFormat;

public class AudioDataUtil {

	/**
	 * Reads from the reader over the full data array or until the reader returns -1 
	 */
	public static int readFully(AudioDataReader reader, int[] data) throws IOException {
		int dataLength = data.length;
		int allRead = -1;
		int unread = dataLength;
		int[] buffer = data;
		do {
			int read = reader.read(buffer);
			if (read == -1) {
				return allRead;
			}
			if (allRead == -1) {
				allRead = 0;
			} else {
				System.arraycopy(buffer, 0, data, allRead, read);
			}
			allRead += read;
			unread = dataLength - allRead;
			buffer = new int[unread];
		} while (unread > 0);
		return allRead;
	}
	
	public static AudioFormat createCDAudioFormat() {
		int sampleRate = 44100;
		int sampleSizeInBits = 16;
		int channels = 2;
		int frameSizeInBytes = sampleSizeInBits * channels / 8;
		int frameRate = sampleRate; // with the above frameSize the rate of
									// frames and samples is identical
		AudioFormat format = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, sampleRate, sampleSizeInBits, channels, frameSizeInBytes, frameRate, true);
		return format;
	}
	
	public static void dumpAudioDataReaderChain(AudioDataReader reader, Appendable a) throws IOException {
		if (reader instanceof Dumpable) {
			((Dumpable) reader).dump(a);
		} else {
			a.append(String.valueOf(reader)).append('\n');
			if (reader instanceof AudioDataReaderChainLink) {
				AudioDataReader source = ((AudioDataReaderChainLink) reader).getSource();
				dumpAudioDataReaderChain(source, a);
			}
		}
	}
}
