/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

public class Delay {
	private final int delayBufferSize;
	private final int[] delayBuffer;
	private int bufferIndex;

	public Delay(float frameRate, int delayTimeInMilliseconds) {
		delayBufferSize = (int) (delayTimeInMilliseconds * frameRate / 1000F);
		delayBuffer = new int[delayBufferSize];
	}

	public Delay(int delayBufferSize) {
		this.delayBufferSize = delayBufferSize;
		delayBuffer = new int[delayBufferSize];
	}
	
	int[] delay(int[] data) {
		return delay(data, data.length);
	}
	
	int[] delay(int[] data, int length) {
		delay(data, 0, length);
		return data;
	}
	
	public void delay(int[] data, int offset, int length) {
		for (int i = offset; i < length; i ++) {
			if (delayBufferSize > 0) {
				int delayedValue = delayBuffer[bufferIndex];
				// remember new value
				delayBuffer[bufferIndex] = data[i];
				// return any delayed previous value
				data[i] = delayedValue;
				bufferIndex++;
				if (bufferIndex >= delayBufferSize) {
					bufferIndex = 0;
				}
			}
		}
	}
}
