/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

import java.io.IOException;

public class DelegatingAudioDataReader implements AudioDataReaderChainLink, Omitable {
	
	protected AudioDataReader reader;
	
	public DelegatingAudioDataReader(AudioDataReader reader) {
		if(reader==null) {
			throw new IllegalArgumentException("Reader must not be null.");
		}
		this.reader = reader;
	}

	@Override
	public int read(int[] data) throws IOException {
		return reader.read(data);
	}

	@Override
	public void close() throws IOException {
		reader.close();
	}

	@Override
	public void stop() {
		reader.stop();
	}

	@Override
	public void seek(long framePosition) throws IOException {
		reader.seek(framePosition);
	}

	@Override
	public void setSource(AudioDataReader source) {
		this.reader = source;
	}

	@Override
	public AudioDataReader getSource() {
		return reader;
	}

	@Override
	public void omit(boolean omit) {
		if(reader instanceof Omitable) {
			((Omitable)reader).omit(true);
		}
	}

}
