/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;

public class InputStreamAudioDataReader extends UnifiedAudioDataReader {

	private final InputStream inputStream;
	private final AudioFormat format;
	private boolean stopped;
	
	public InputStreamAudioDataReader(File file) throws IOException {
		try {
			AudioInputStream audioInputStream = AudioSystem.getAudioInputStream(file);
			this.inputStream = audioInputStream;
			this.format = audioInputStream.getFormat();
		} catch (UnsupportedAudioFileException e) {
			throw new IllegalStateException(e);
		}
	}

	public InputStreamAudioDataReader(AudioInputStream audioInputStream) {
		this.inputStream = audioInputStream;
		this.format = audioInputStream.getFormat();
	}

	public InputStreamAudioDataReader(InputStream inputStream, AudioFormat format) {
		this.inputStream = inputStream;
		this.format = format;
	}

	@Override
	protected int read(byte[] data, int offset, int length) throws IOException {
		if (stopped) {
			return -1;
		}
		return inputStream.read(data, offset, length);
	}

	@Override
	public void close() throws IOException {
		inputStream.close();
		super.close();
	}

	@Override
	public void stop() {
		stopped = true;
	}

	@Override
	public void seek(long samplePosition) throws IOException {
		throw new UnsupportedOperationException("Seek is not supported by " + getClass().getName());
	}

	@Override
	protected AudioFormat getAudioFormat() throws IOException {
		return format;
	}
}
