/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;


public class IntArrayAudioDataReader implements AudioDataReader {
	private int[] sourceData;
	private int idx;
	private int remain;
	private boolean stopped;
	private int length;

	public IntArrayAudioDataReader(int[] sourceData) {
		this.sourceData = sourceData;
		this.length = sourceData.length;
		remain = length;
	}

	@Override
	public int read(int[] data) {
		if (remain == 0 || stopped) {
			return -1;
		}
		int read = Math.min(remain, data.length);
		System.arraycopy(sourceData, idx, data, 0, read);
		idx += read;
		remain -= read;
		return read;
	}

	@Override
	public void close() {
		stop();
	}

	@Override
	public void stop() {
		stopped = true;
	}

	@Override
	public void seek(long samplePosition) {
		if (samplePosition >= length || samplePosition < 0 || samplePosition > Integer.MAX_VALUE) {
			throw new IllegalArgumentException("Sample index out of range: " + samplePosition + " (max length is " + length + ")");
		}
		idx = (int) samplePosition;
		remain = length - idx;
	}
}
