/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

import java.io.IOException;


public class SourceSinkAudioDataReader implements AudioDataReaderChainLink, Omitable {
	private AudioDataReaderChainLink sourceReader;
	private AudioDataReader sinkReader;
	
	public SourceSinkAudioDataReader(AudioDataReaderChainLink sourceReader, AudioDataReader sinkReader) {
		this.sourceReader = sourceReader;
		this.sinkReader = sinkReader;
	}

	@Override
	public int read(int[] data) throws IOException {
		return sinkReader.read(data);
	}

	@Override
	public void close() throws IOException {
		sinkReader.close();
	}

	@Override
	public void stop() {
		sinkReader.stop();
	}

	@Override
	public void seek(long framePosition) throws IOException {
		sinkReader.seek(framePosition);
	}

	@Override
	public void setSource(AudioDataReader source) {
		sourceReader.setSource(source);
	}

	@Override
	public AudioDataReader getSource() {
		return sourceReader;
	}

	@Override
	public void omit(boolean omit) {
		if(sourceReader instanceof Omitable) {
			((Omitable)sourceReader).omit(true);
		}
		if(sinkReader instanceof Omitable) {
			((Omitable)sinkReader).omit(true);
		}
	}

}
