/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;

public enum AudioChannels {
	Mono(1, (char) 9900), Stereo(2, (char) 9901);
	private final int channelCount;
	private final char symbol;

	private AudioChannels(int channels, char symbol) {
		this.channelCount = channels;
		this.symbol = symbol;
	}

	public int getChannelCount() {
		return channelCount;
	}

	public static AudioChannels fromChannelCount(int channelCount) {
		for (AudioChannels channel : values()) {
			if (channel.channelCount == channelCount) {
				return channel;
			}
		}
		throw new IllegalArgumentException("The channel count '" + channelCount + "' is not supported.");
	}

	public AudioChannels getToggle() {
		switch (this) {
		case Mono:
			return Stereo;
		case Stereo:
			return Mono;
		}
		// in the case that the channel enum is gonna be extended and the hacker
		// forgets to extend this method
		throw new UnsupportedOperationException("The " + AudioChannels.class.getSimpleName() + " value " + this.name() + " is not supported for toggling yet.");
	}

	public char getSymbol() {
		return symbol;
	}
}
