/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;

import java.util.Collection;

public interface AudioDeviceFactory {
	public static class AudioDeviceIdentifier {
		private final String identifier;
		private final String name;
		private final boolean sourceAudioDevice;

		/**
		 * @param identifier the internal identifier to distinguish from other identifiers
		 * @param name the displayed label for the user
		 * @param sourceAudioDevice defines whether the device is a source device (true) or a chainable audio device (false).
		 */
		public AudioDeviceIdentifier(String identifier, String name, boolean sourceAudioDevice) {
			if (identifier == null) {
				throw new IllegalArgumentException("identifier must not be null");
			}
			if (name == null) {
				throw new IllegalArgumentException("name must not be null");
			}
			this.identifier = identifier;
			this.name = name;
			this.sourceAudioDevice = sourceAudioDevice;
		}

		public String getIdentifier() {
			return identifier;
		}

		public String getName() {
			return name;
		}

		public boolean isSourceAudioDevice() {
			return sourceAudioDevice;
		}

		@Override
		public String toString() {
			return name;
		}
	}

	public AudioDevice createAudioDevice(AudioDeviceIdentifier audioDeviceIdentifier);
	public Collection<AudioDeviceIdentifier> getAudioDeviceIdentifiers();
}
