/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;

/**
 * This listener is used to become informed when an {@link AudioDevice} changes.
 */
public interface AudioDeviceListener {
	/**
	 * Will be fired by any {@link AudioDevice} when inner properties became changed
	 * that will be potentially audible by the user. These changes will be synced to remote
	 * projects.
	 * 
	 * @param audioDevice
	 */
	public void performanceRelevantAudioDevicePropertyChanged(AudioDevice audioDevice);
	
	/**
	 * Will be fired by any {@link AudioDevice} when properties become changed that are
	 * persistable and must be saved with the project in a project file. This will also be fired when the
	 * performanceRelevantAudioDevicePropertyChanged() is fired.
	 * 
	 * @param audioDevice
	 */
	public void persistableAudioDevicePropertyChanged(AudioDevice audioDevice);
}
