/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;

public interface BinaryDataset {
	public AudioProject getBinaryDatasetAssociatedAudioProject();
	/**
	 * The binary dataset is identified globally by this identifier. Each binary
	 * dataset (files, object instances, etc) must have an unique identifier.
	 * 
	 * @return any identifier that is used to distinguish each binary dataset
	 *         from the other and to identify what to create in the
	 *         processBinaryDataset method.
	 */
	public String getBinaryDatasetIdentifier();
	public InputStream getBinaryDatasetInputStream() throws IOException;
	public long getBinaryDatasetLength() throws IOException;
	public void processBinaryDataset(AudioProject associatedAudioProject, String binaryDatasetIdentifier, InputStream is, long length, BinaryDatasetProcessCallback callback) throws IOException, IllegalAccessException,
			ClassNotFoundException, InstantiationException, NoSuchMethodException, InvocationTargetException;
	
	public static interface BinaryDatasetProcessCallback {
		public void binaryDatasetProcessUpdate(long processedBytes, long totalBytes);
	}
}
