/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;

public class BooleanAudioDeviceValue extends AudioDeviceValue {
	public boolean value;
	private char falseSymbol;

	public BooleanAudioDeviceValue(AudioDevice audioDevice, String name, boolean value) {
		super(audioDevice, name);
		this.value = value;
	}

	public boolean getValue() {
		return value;
	}

	public void setValue(boolean value) {
		boolean oldValue = this.value;
		this.value = value;
		if (oldValue != value) {
			fireAudioDeviceValueChanged(String.valueOf(oldValue), String.valueOf(value));
		}
	}

	@Override
	protected void copy(AudioDeviceValue from) {
		if (!getName().equals(from.getName())) {
			throw new IllegalArgumentException("The " + AudioDeviceValue.class.getSimpleName() + " must have the same name. This name '" + getName() + "', the other name to be copied from is '"
					+ from.getName() + "'.");
		}
		setValue(((BooleanAudioDeviceValue) from).value);
	}

	@Override
	public void setValueAsString(String stringValue) {
		setValue(Boolean.valueOf(stringValue));
	}

	@Override
	public String getValueAsString() {
		return String.valueOf(value);
	}

	@Override
	public String getDisplayString() {
		return String.valueOf(getValue() ? getSymbol() : getFalseSymbol());
	}

	@Override
	public String getHelpString() {
		String defaultValue = getDisplayString();
		setValue(true);
		String trueStr = getDisplayString();
		setValue(false);
		String falseStr = getDisplayString();
		return format(defaultValue, "true: " + trueStr + ", false: " + falseStr);
	}

	public char getFalseSymbol() {
		return falseSymbol;
	}

	public void setFalseSymbol(char falseSymbol) {
		this.falseSymbol = falseSymbol;
	}
}
