/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;

import org.audiochain.io.AudioDataReader;
import org.audiochain.io.AudioDataReaderChainLink;
import org.audiochain.io.FrameObserver;

public interface ChainableAudioDevice extends AudioDevice {

	/**
	 * Enables the AudioDevice. When a new track is created, all registered enabled AudioControls will be added to the track.
	 * All disabled AudioControls can be added to the track and enabled manually by the user at runtime.
	 * 
	 * @param enabled true, or false if the AudioDevice should be disabled.
	 */
	public void setEnabled(boolean enabled);
	
	/**
	 * @return true, if the AudioDevice is enabled, false if it is disabled.
	 */
	public boolean isEnabled();
	public AudioDataReaderChainLink createAudioDataReaderChainLink(AudioDataReader source, float frameRate, int sampleSizeInBits, FrameObserver frameObserver);
	
	/**
	 * The implementation should use a Set for internal listener Collection, because this method may be called multiple times with the same listener instance as parameter.
	 * @param listener the listener instance that may be potentially be added multiple times.
	 */
	public void addChainableAudioDeviceListener(ChainableAudioDeviceListener listener);
	public void removeChainableAudioDeviceListener(ChainableAudioDeviceListener listener);
}
