/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;

import java.util.Arrays;
import java.util.Collection;

public class EnumAudioDeviceValue<T extends Enum<T>> extends AudioDeviceValue {
	private T value;
	private Collection<T> options;
	private Class<T> enumClass;

	public EnumAudioDeviceValue(AudioDevice audioDevice, String name, T value, Class<T> enumClass, T... options) {
		super(audioDevice, name);
		this.enumClass = enumClass;
		setOptions(options);
		this.value = value;
	}

	@Override
	protected void copy(AudioDeviceValue from) {
		@SuppressWarnings("unchecked")
		EnumAudioDeviceValue<T> other = (EnumAudioDeviceValue<T>) from;
		this.options = other.options;
		this.enumClass = other.enumClass;
		setValue(other.value);
	}

	public T getValue() {
		return value;
	}

	public void setValue(T value) {
		T oldValue = this.value;
		this.value = value;
		if (!isEqual(oldValue, value)) {
			fireAudioDeviceValueChanged(oldValue != null ? oldValue.name() : null, getValueAsString());
		}
	}

	public Collection<T> getOptions() {
		return options;
	}

	public void setOptions(Collection<T> options) {
		this.options = options;
	}

	public void setOptions(T... options) {
		this.options = Arrays.asList(options);
	}

	@Override
	public void setValueAsString(String stringValue) {
		T value;
		if (stringValue == null) {
			value = null;
		} else {
			value = Enum.valueOf(enumClass, stringValue);
		}
		setValue(value);
	}

	@Override
	public String getValueAsString() {
		if (value != null) {
			return value.name();
		}
		return null;
	}

	public Class<T> getEnumClass() {
		return enumClass;
	}

	public void setEnumClass(Class<T> enumClass) {
		this.enumClass = enumClass;
	}

	@Override
	public String getHelpString() {
		return format(getDisplayString(), String.valueOf(getOptions()));
	}
}
