/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;


public class FloatAudioDeviceValue extends AudioDeviceValue {
	private float value;
	private Float maxValue;
	private Float minValue;
	private Float midValue;
	private Scale scale;

	public FloatAudioDeviceValue(AudioDevice audioDevice, String name, float value) {
		super(audioDevice, name);
		this.value = value;
	}

	public float getValue() {
		return value;
	}

	public void setValue(float value) {
		float oldValue = this.value;
		this.value = value;
		if (oldValue != value) {
			fireAudioDeviceValueChanged(String.valueOf(oldValue), String.valueOf(value));
		}
	}

	@Override
	protected void copy(AudioDeviceValue from) {
		FloatAudioDeviceValue other = (FloatAudioDeviceValue) from;
		this.maxValue = other.maxValue;
		this.minValue = other.minValue;
		setValue(other.value);
	}

	@Override
	public void setValueAsString(String stringValue) {
		float newValue = Float.parseFloat(stringValue);
		if (minValue != null && newValue < minValue) {
			throw new IllegalArgumentException("The value (" + newValue + ") must not be smaller than " + minValue + ".");
		}
		if (maxValue != null && newValue > maxValue) {
			throw new IllegalArgumentException("The value (" + newValue + ") must not be greater than " + maxValue + ".");
		}
		setValue(newValue);
	}

	@Override
	public String getValueAsString() {
		return String.valueOf(value);
	}

	public Float getMaxValue() {
		return maxValue;
	}

	public void setMaxValue(Float maxValue) {
		this.maxValue = maxValue;
	}

	public Float getMinValue() {
		return minValue;
	}

	public void setMinValue(Float minValue) {
		this.minValue = minValue;
	}

	public Float getMidValue() {
		return midValue;
	}

	public void setMidValue(Float midValue) {
		this.midValue = midValue;
	}

	public Scale getScale() {
		return scale;
	}

	public void setScale(Scale scale) {
		this.scale = scale;
	}

	@Override
	public String getHelpString() {
		Float min = getMinValue();
		Float max = getMaxValue();
		String minStr = min == null ? "undefined" : String.valueOf(min);
		String maxStr = max == null ? "undefined" : String.valueOf(max);
		return format(getDisplayString(), "min: " + minStr + ", max: " + maxStr);
	}
}
