/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;

import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Mixer;

/**
 * This Class provides two capabilities. The first is the descriptive String
 * representation of a hardware mixer, the second capability is the object
 * comparison (equals and hashCode implementation by the mixers name).
 */
public class HardwareMixer {
	private final Mixer.Info mixerInfo;

	HardwareMixer(Mixer.Info mixerInfo) {
		if (mixerInfo == null) {
			throw new IllegalArgumentException("Mixer Info must not be null.");
		}
		this.mixerInfo = mixerInfo;
	}

	public String getName() {
		return mixerInfo.getName();
	}

	public String getDescription() {
		return mixerInfo.getName() + " " + mixerInfo.getDescription() + " " + mixerInfo.getVendor() + " " + mixerInfo.getVersion();
	}

	Mixer.Info getMixerInfo() {
		return mixerInfo;
	}

	public Mixer getMixer() {
		return AudioSystem.getMixer(mixerInfo);
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof HardwareMixer)) {
			return false;
		}
		HardwareMixer other = (HardwareMixer) o;
		return getName().equals(other.getName());
	}

	@Override
	public int hashCode() {
		return getName().hashCode();
	}

	@Override
	public String toString() {
		return getDescription();
	}
}
