/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;


public class IntAudioDeviceValue extends AudioDeviceValue {
	private int value;
	private Integer maxValue;
	private Integer minValue;
	private Integer midValue;

	public IntAudioDeviceValue(AudioDevice audioDevice, String name, int value) {
		super(audioDevice, name);
		this.value = value;
	}

	public int getValue() {
		return value;
	}

	public void setValue(int value) {
		int oldValue = this.value;
		this.value = value;
		if (oldValue != value) {
			fireAudioDeviceValueChanged(String.valueOf(oldValue), String.valueOf(value));
		}
	}

	@Override
	protected void copy(AudioDeviceValue from) {
		IntAudioDeviceValue other = (IntAudioDeviceValue) from;
		this.maxValue = other.maxValue;
		this.minValue = other.minValue;
		setValue(other.value);
	}

	@Override
	public void setValueAsString(String stringValue) {
		int newValue = Integer.parseInt(stringValue);
		if (minValue != null && newValue < minValue) {
			throw new IllegalArgumentException("The value (" + newValue + ") must not be smaller than " + minValue + ".");
		}
		if (maxValue != null && newValue > maxValue) {
			throw new IllegalArgumentException("The value (" + newValue + ") must not be greater than " + maxValue + ".");
		}
		setValue(newValue);
	}

	@Override
	public String getValueAsString() {
		return String.valueOf(value);
	}

	public Integer getMaxValue() {
		return maxValue;
	}

	public void setMaxValue(Integer maxValue) {
		this.maxValue = maxValue;
	}

	public Integer getMinValue() {
		return minValue;
	}

	public void setMinValue(Integer minValue) {
		this.minValue = minValue;
	}

	public Integer getMidValue() {
		return midValue;
	}

	public void setMidValue(Integer midValue) {
		this.midValue = midValue;
	}

	@Override
	public String getHelpString() {
		Integer min = getMinValue();
		Integer max = getMaxValue();
		return format(getDisplayString(), "min: " + (min == null ? "undefined" : String.valueOf(min)) + ", max: " + (max == null ? "undefined" : String.valueOf(max)));
	}
}
