/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.model;

import java.io.IOException;

import org.audiochain.io.AudioDataReader;
import org.audiochain.io.AudioDataReaderChainLink;
import org.audiochain.io.AudioDataUtil;
import org.audiochain.io.Dumpable;

class PerformanceMonitor implements AudioDataReaderChainLink, Monitorable, Dumpable {
	private int[] monitorAudioData;
	private int readLength;
	private boolean monitorEnabled;
	private AudioDataReader reader;

	public PerformanceMonitor(AudioDataReader reader) {
		this.reader = reader;
	}

	@Override
	public int read(int[] data) throws IOException {
		readLength = reader.read(data);
		if (monitorEnabled) {
			if (monitorAudioData == null) {
				monitorAudioData = new int[data.length];
			}
			if (readLength > 0) {
				// may have been resized because capure buffer is smaller than recording buffer (see Recording)
				if (readLength > monitorAudioData.length) {
					monitorAudioData = new int[data.length];
				}
				System.arraycopy(data, 0, monitorAudioData, 0, readLength);
			}
		}
		return readLength;
	}

	@Override
	public int[] getMonitorData() {
		return monitorAudioData;
	}

	@Override
	public int getMonitorLength() {
		return readLength;
	}

	@Override
	public boolean isMonitorEnabled() {
		return monitorEnabled;
	}

	@Override
	public void setMonitorEnabled(boolean monitorEnabled) {
		this.monitorEnabled = monitorEnabled;
	}
	
	@Override
	public void setSource(AudioDataReader reader) {
		if (this.reader instanceof AudioDataReaderChainLink) {
			((AudioDataReaderChainLink) this.reader).setSource(reader);
		} else {
			this.reader = reader;
		}
	}

	@Override
	public AudioDataReader getSource() {
		if (reader instanceof AudioDataReaderChainLink) {
			return ((AudioDataReaderChainLink) reader).getSource();
		}
		return reader;
	}

	@Override
	public void stop() {
		reader.stop();
	}

	@Override
	public void seek(long framePosition) throws IOException {
		reader.seek(framePosition);		
	}

	@Override
	public void close() throws IOException {
		reader.close();
	}
	
	@Override
	public void dump(Appendable a) throws IOException {
		a.append(String.valueOf(this)).append(monitorEnabled ? " enabled" : "").append('\n');
		AudioDataUtil.dumpAudioDataReaderChain(reader, a);
	}
}
