/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui;

import java.util.Collection;
import java.util.LinkedHashSet;

import org.audiochain.model.AudioProject;
import org.audiochain.model.AudioProjectLifecycleListener;
import org.audiochain.model.AudioProjectLifecycleObservable;

public class AudioProjectLifecycleObserver implements AudioProjectLifecycleObservable {
	private Collection<AudioProjectLifecycleListener> listeners;
	private Collection<AudioProject> registeredAudioProjects;
	
	@Override
	public void registerAudioProject(AudioProject audioProject) {
		if (registeredAudioProjects == null) {
			registeredAudioProjects = new LinkedHashSet<AudioProject>();
		}
		registeredAudioProjects.add(audioProject);
	}
	
	@Override
	public void unregisterAudioProject(AudioProject audioProject) {
		if (registeredAudioProjects == null) {
			return;
		}
		registeredAudioProjects.remove(audioProject);
		if (registeredAudioProjects.isEmpty()) {
			registeredAudioProjects = null;
		}
	}

	@Override
	public Collection<AudioProject> getAudioProjects() {
		LinkedHashSet<AudioProject> ret = new LinkedHashSet<AudioProject>();
		if (registeredAudioProjects != null) {
			ret.addAll(registeredAudioProjects);
		}
		return ret;
	}

	protected void fireAudioProjectOpened(AudioProject audioProject) {
		if (listeners == null) {
			return;
		}
		audioProject.fireAudioProjectOpened();
		for (AudioProjectLifecycleListener listener : new LinkedHashSet<AudioProjectLifecycleListener>(listeners)) {
			listener.audioProjectOpened(audioProject);
		}
	}

	protected void fireAudioProjectClosed(AudioProject audioProject) {
		if (listeners == null) {
			return;
		}
		audioProject.fireAudioProjectClosed();
		for (AudioProjectLifecycleListener listener : new LinkedHashSet<AudioProjectLifecycleListener>(listeners)) {
			listener.audioProjectClosed(audioProject);
		}
	}

	protected void fireAudioProjectStored(AudioProject audioProject) {
		if (listeners == null) {
			return;
		}
		audioProject.fireAudioProjectStored();
		for (AudioProjectLifecycleListener listener : new LinkedHashSet<AudioProjectLifecycleListener>(listeners)) {
			listener.audioProjectStored(audioProject);
		}
	}

	protected void fireAudioProjectActivated(AudioProject audioProject) {
		if (listeners == null) {
			return;
		}
		if(audioProject!=null) {
			audioProject.fireAudioProjectActivated();
		}
		for (AudioProjectLifecycleListener listener : new LinkedHashSet<AudioProjectLifecycleListener>(listeners)) {
			listener.audioProjectActivated(audioProject);
		}
	}

	@Override
	public void addAudioProjectLifecycleListener(AudioProjectLifecycleListener listener) {
		if (listeners == null) {
			listeners = new LinkedHashSet<AudioProjectLifecycleListener>();
		}
		listeners.add(listener);
	}

	@Override
	public void removeAudioProjectLifecycleListener(AudioProjectLifecycleListener listener) {
		if (listeners == null) {
			return;
		}
		listeners.remove(listener);
	}
}
