/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui;

import java.util.Collection;

import org.audiochain.model.AbstractCommandLineUserInterface;
import org.audiochain.model.AudioDevice;
import org.audiochain.model.AudioDeviceValue;

public class GenericCommandLineUserInterface extends AbstractCommandLineUserInterface {
	private final AudioDevice audioDevice;

	public GenericCommandLineUserInterface(AudioDevice audioDevice) {
		this.audioDevice = audioDevice;
	}

	@Override
	public String getDisplayString() {
		StringBuilder sb = new StringBuilder();
		boolean first = true;
		Collection<AudioDeviceValue> audioDeviceValues = audioDevice.getAudioDeviceValues();
		if (audioDeviceValues != null) {
			for (AudioDeviceValue value : audioDeviceValues) {
				String displayString = value.getDisplayString();
				if (displayString != null) {
					if (!first) {
						sb.append(" ");
					}
					sb.append(displayString);
					first = false;
				}
			}
		}
		return sb.toString();
	}

	@Override
	public String getHelpString() {
		StringBuilder sb = new StringBuilder();
		Collection<AudioDeviceValue> audioDeviceValues = audioDevice.getAudioDeviceValues();
		if (audioDeviceValues != null) {
			for (AudioDeviceValue val : audioDeviceValues) {
				String helpString = val.getHelpString();
				if (helpString != null) {
					sb.append(helpString);
				}
			}
		}
		return sb.toString();
	}

	@Override
	public boolean modifyAudioDevice(String propertyName, String newValue) throws Exception {
		String name = propertyName.trim().toLowerCase();
		for (AudioDeviceValue value : audioDevice.getAudioDeviceValues()) {
			if (value.getPropertyName().toLowerCase().startsWith(name)) {
				value.setValueAsString(newValue);
				return true;
			}
		}
		return false;
	}
}
