/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

public class LightEmittingDiodeMatrix {
	private final List<LightEmittingDiodeArray> ledArrays;
	private final int arrayCount;
	private final int[] arrayBaseColors;
	private final int ledWidth;
	private final int ledHeight;

	public LightEmittingDiodeMatrix(int arrayCount, int[] arrayBaseColors, int ledWidth, int ledHeight) {
		this.ledArrays = new ArrayList<LightEmittingDiodeArray>();
		this.arrayCount = arrayCount;
		this.arrayBaseColors = arrayBaseColors;
		this.ledWidth = ledWidth;
		this.ledHeight = ledHeight;
		init();
	}

	private void init() {
		for (int i = 0; i < arrayCount; i++) {
			LightEmittingDiodeArray ledArray = new LightEmittingDiodeArray(arrayBaseColors, ledWidth, ledHeight);
			ledArrays.add(ledArray);
		}
	}

	public Collection<LightEmittingDiodeArray> getLedArrays() {
		return Collections.unmodifiableCollection(ledArrays);
	}

	public void setLedArrayValue(int arrayIndex, float value) {
		ledArrays.get(arrayIndex).setLedValue(value);
	}

	public void setLedArraySingleValue(int arrayIndex, float value) {
		ledArrays.get(arrayIndex).setLedSingleValue(value);
	}

	public void setLedArrayValue(int arrayIndex, float value, float singleValue) {
		LightEmittingDiodeArray lightEmittingDiodeArray = ledArrays.get(arrayIndex);
		lightEmittingDiodeArray.setLedValue(value);
		lightEmittingDiodeArray.setLedSingleValue(singleValue);
	}

	public LightEmittingDiodeArray getLightEmittingDiodeArray(int arrayIndex) {
		return ledArrays.get(arrayIndex);
	}
}
