/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.cli;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

abstract public class PatternCommand implements Command {
	protected Matcher m;
	private Pattern pattern;
	private final String syntax;
	private final String description;

	public PatternCommand(String syntax, String description) {
		this.syntax = syntax;
		this.description = description;
	}

	@Override
	public boolean matches(String command) {
		if (pattern == null) {
			String pat = getPattern();
			if (pat != null) {
				pattern = Pattern.compile(pat);
			}
		}
		if (pattern != null) {
			m = pattern.matcher(command);
			boolean matches = m.matches();
			if (matches) {
				return true;
			}
			m = null;
			return false;
		}
		return command.equals(getSyntax());
	}

	protected String getPattern() {
		return null;
	}

	@Override
	public final String getDescription() {
		return description;
	}

	@Override
	public final String getSyntax() {
		return syntax;
	}
}
