/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui;

import static org.audiochain.ui.gui.BackgroundImageComponent.*;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBox;

import org.audiochain.model.AudioChannels;


public class AudioChannelsCheckBox extends JCheckBox {
	private static final long serialVersionUID = 1L;

	public AudioChannelsCheckBox() {
		this(null, AudioChannels.Mono);
	}

	public AudioChannelsCheckBox(String text) {
		this(text, AudioChannels.Mono);
	}

	public AudioChannelsCheckBox(String text, AudioChannels channels) {
		super(text, channels == AudioChannels.Stereo);
		init();
	}

	private void init() {
		setOpaque(false);
		setIcon(readIcon("org/audiochain/ui/gui/mono-item.png"));
		setSelectedIcon(readIcon("org/audiochain/ui/gui/stereo-item.png"));
		setToolTipText(getAudioChannels().name());
		addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				setToolTipText(getAudioChannels().name());
			}
		});
	}

	public void setAudioChannels(AudioChannels channels) {
		setSelected(channels == AudioChannels.Stereo);
	}

	public AudioChannels getAudioChannels() {
		return isSelected() ? AudioChannels.Stereo : AudioChannels.Mono;
	}
}
