/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui;

import java.awt.Graphics;
import java.awt.Image;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

/**
 * This Component tiles the given background image.
 * 
 * To create a background with gimp:
 * <li> create a square picture of 250px with transparent background
 * <li> fill the color with rgb values 25,25,25
 * <li> filter: rgb-distortion with values rgb 0,0,0 alpha 0,3 and non correlated
 */
public class BackgroundImageComponent extends JPanel {
	private static final long serialVersionUID = 1L;
	private static Map<String, ImageIcon> iconCache = new HashMap<String, ImageIcon>();

	final ImageIcon background;

	public BackgroundImageComponent(String imagePath) {
		background = readIcon(imagePath);
	}

	@Override
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Image image = background.getImage();
		int iw = image.getWidth(null);
		int ih = image.getHeight(null);
		int w = getWidth();
		int h = getHeight();
		for (int x = 0; x < w; x += iw) {
			for (int y = 0; y < h; y += ih) {
				g.drawImage(image, x, y, null);
			}
		}
	}

	public static ImageIcon readIcon(String path) {
		ImageIcon icon = iconCache.get(path);
		if(icon==null) {
			if (!path.startsWith("/") && path.contains("/")) {
				path = "/" + path;
			}
			InputStream is = BackgroundImageComponent.class.getResourceAsStream(path);
			if (is == null) {
				throw new IllegalArgumentException("Resource '" + path + "' does not exist.");
			}
			byte[] bytes = new byte[1024];
			int len;
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			try {
				while ((len = is.read(bytes)) != -1) {
					baos.write(bytes, 0, len);
				}
				is.close();
				baos.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			bytes = baos.toByteArray();
	
			icon = new ImageIcon(bytes);
			iconCache.put(path, icon);
		}
		return icon;
	}
}
