/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui;

import static javax.swing.SwingConstants.*;
import static org.audiochain.ui.gui.GlobalGuiContext.*;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.util.Arrays;
import java.util.Collection;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.plaf.ScrollBarUI;
import javax.swing.plaf.basic.BasicComboBoxRenderer;
import javax.swing.plaf.basic.BasicComboBoxUI;
import javax.swing.plaf.basic.BasicScrollBarUI;

public class CommonComponentFactory {
	
	public static Component createHorizontalLine() {
		return createLine(true);
	}
	
	public static Component createVerticalLine() {
		return createLine(false);
	}
	
	private static Component createLine(boolean horizontal) {
		Box.Filler line = new Box.Filler(new Dimension(1, 1), new Dimension(1, 1), horizontal ? new Dimension(Short.MAX_VALUE, 1) : new Dimension(1, Short.MAX_VALUE));
		line.setBackground(getGuiSettings().getBackgroundColor());
		line.setOpaque(true);
		return line;
	}

	public static JScrollPane createScrollPane(Component comp) {
		JScrollPane scrollPane = new JScrollPane(comp);
		scrollPane.setOpaque(false);
		scrollPane.getViewport().setOpaque(false);
		scrollPane.setBorder(null);
		JScrollBar horizontalScrollBar = scrollPane.getHorizontalScrollBar();
		horizontalScrollBar.setUnitIncrement(15);
		horizontalScrollBar.setOpaque(false);
		JScrollBar verticalScrollBar = scrollPane.getVerticalScrollBar();
		verticalScrollBar.setOpaque(false);
		verticalScrollBar.setUnitIncrement(15);
	    horizontalScrollBar.setUI(createScrollBarUI());
		verticalScrollBar.setUI(createScrollBarUI());
		return scrollPane;
	}
	
	public static ScrollBarUI createScrollBarUI() {
		final int scrollBarWidth = 8;
		final Color backgroundColor = getGuiSettings().getLightBackgroundColor();
		final JButton zeroButton = new JButton();
		zeroButton.setMaximumSize(new Dimension(0, 0));
		zeroButton.setPreferredSize(zeroButton.getMaximumSize());
		zeroButton.setMinimumSize(zeroButton.getMaximumSize());
		ScrollBarUI scrollBarUi = new BasicScrollBarUI() {
			@Override
			protected void paintThumb(Graphics g, JComponent c, Rectangle thumbBounds) {
				g.translate(thumbBounds.x, thumbBounds.y);
				g.setColor(backgroundColor);
				g.fillRect(0, 0, thumbBounds.width - 1, thumbBounds.height - 1);
				g.translate(-thumbBounds.x, -thumbBounds.y);
			}

			@Override
			protected void paintTrack(Graphics g, JComponent c, Rectangle trackBounds) {
				// no track displayed
			}

			@Override
			protected JButton createDecreaseButton(int orientation) {
				return zeroButton;
			}

			@Override
			protected JButton createIncreaseButton(int orientation) {
				return zeroButton;
			}

			@Override
			public Dimension getPreferredSize(JComponent c) {
				return (scrollbar.getOrientation() == VERTICAL) ? new Dimension(scrollBarWidth, 48) : new Dimension(48, scrollBarWidth);
		    }
		};
		return scrollBarUi;
	}
	
	public static JComboBox createComboBox() {
		return createComboBox((ComboBoxModel) null);
	}
	
	public static JComboBox createComboBox(Object... data) {
		return data == null ? createComboBox((ComboBoxModel) null) : createComboBox(Arrays.asList(data));
	}

	public static JComboBox createComboBox(Collection<?> data) {
		return data == null ? createComboBox((ComboBoxModel) null) : createComboBox(new DefaultComboBoxModel(new Vector<Object>(data)));
	}
	
	public static JComboBox createComboBox(ComboBoxModel model) {
		JComboBox ret = model == null ? new JComboBox() : new JComboBox(model);
		// hide the drop down button to make it look like a label
		ret.setUI(new BasicComboBoxUI() {
			@Override
			protected JButton createArrowButton() {
				JButton ret = new JButton() {
					private static final long serialVersionUID = 1L;

					@Override
					public int getWidth() {
						return 0;
					}
				};
				return ret;
			}
		});

		ListCellRenderer renderer = ret.getRenderer();
		if (renderer instanceof BasicComboBoxRenderer) {
			((BasicComboBoxRenderer) renderer).setHorizontalAlignment(CENTER);
		}

		ret.setFocusable(false);
		ret.setMaximumSize(new Dimension(Integer.MAX_VALUE, ret.getPreferredSize().height));
		return ret;
	}
}
