/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import org.audiochain.model.AudioProject;


public class GlobalGuiContext {
	private GuiSettings settings;
	private AudioProject projectTemplate;
	
	private GlobalGuiContext() {
		
	}
	
	public static GlobalGuiContext getInstance() {
		return GlobalGuiContextHolder.instance;
	}
	
	private static class GlobalGuiContextHolder {
		private static GlobalGuiContext instance = new GlobalGuiContext();
	}
	
	public static GuiSettings getGuiSettings() {
		return getInstance().getSettings();
	}

	public GuiSettings getSettings() {
		if(settings==null) {
			settings = GuiSettings.load();
		}
		return settings;
	}

	public AudioProject getProjectTemplate() {
		if(projectTemplate==null) {
			String projectTemplateFileName = getSettings().getProjectTemplateFileName();
			if(projectTemplateFileName==null) {
				return null;
			}
			File templateFile = new File(projectTemplateFileName);
			if(!templateFile.exists() || !templateFile.canRead()) {
				return null;
			}
			try {
				projectTemplate = AudioProject.load(templateFile);
			} catch (IOException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}
		}
		return projectTemplate;
	}

	public void setProjectTemplate(AudioProject projectTemplate) {
		this.projectTemplate = projectTemplate;
	}
}
