/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui;

import static org.audiochain.ui.gui.BackgroundImageComponent.*;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.border.Border;

public class ImageButtonComponent extends JComponent {
	private static final long serialVersionUID = 1L;
	private boolean selected;
	private Border releasedBorder = BorderFactory.createRaisedBevelBorder();
	private Border pressedBorder = BorderFactory.createLoweredBevelBorder();
	private final ImageIcon image;

	public ImageButtonComponent(String imagePath) {
		this.image = readIcon(imagePath);
		init();
	}
	
	private void init() {
		setOpaque(false);
		setFocusable(true);
		setDoubleBuffered(true);

		setBorder(releasedBorder);
		addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				requestFocus();
				if(!isSelected()) {
					setBorder(pressedBorder);
				}
			}
			@Override
			public void mouseReleased(MouseEvent e) {
				if(!isSelected()) {
					setBorder(releasedBorder);
				}
			}
		});
		Image img = image.getImage();
		int iw = img.getWidth(null);
		int ih = img.getHeight(null);
		Dimension dim = new Dimension(iw, ih);
		setMinimumSize(dim);
		setPreferredSize(dim);
		setMaximumSize(dim);
	}

	public boolean isSelected() {
		return selected;
	}

	public void setSelected(boolean selected) {
		this.selected = selected;
		setBorder(selected?pressedBorder:releasedBorder);
	}

	public Border getReleasedBorder() {
		return releasedBorder;
	}

	public void setReleasedBorder(Border releasedBorder) {
		this.releasedBorder = releasedBorder;
		setBorder(releasedBorder);
	}

	public Border getPressedBorder() {
		return pressedBorder;
	}

	public void setPressedBorder(Border pressedBorder) {
		this.pressedBorder = pressedBorder;
	}
	
	@Override
	public void paint(Graphics g) {
		super.paint(g);
		g.drawImage(image.getImage(), 0, 0, null);
	}
}
