/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui;

import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

public class ImageSliderComponent extends MoveableImageComponent {
	private static final long serialVersionUID = 1L;
	private int value;
	private int maximum = 100;
	private int minimum;
	private double valueInPercent;

	public ImageSliderComponent(String svgDocumentFileInClasspath) {
		super(svgDocumentFileInClasspath);
		init();
	}

	private void init() {
		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				if (isVisible()) {
					propagateCurrentValueToUI();
				}
			}
		});
		MouseAdapter mouseAdapter = new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				if (!isEnabled()) {
					return;
				}
				gatherValueInPercent();
				showValueLabel(String.valueOf(value), e.getX()+12, e.getY()-12);
			}

			@Override
			public void mouseDragged(MouseEvent e) {
				if (!isEnabled()) {
					return;
				}
				gatherValueInPercent();
				updateValueLabel(String.valueOf(value), e.getX()+12, e.getY()-12);
			}

			@Override
			public void mouseReleased(MouseEvent e) {
				hideValueLabel();
			}
		};

		addMouseListener(mouseAdapter);
		addMouseMotionListener(mouseAdapter);
	}

	public int getValue() {
		return value;
	}

	public void setValue(int value) {
		if (this.value == value) {
			return;
		}
		if (value > maximum) {
			value = maximum;
		} else if (value < minimum) {
			value = minimum;
		}
		this.value = value;
		calculatePercentByValue();
	}

	public int getMaximum() {
		return maximum;
	}

	public void setMaximum(int maximum) {
		this.maximum = maximum;
		calculatePercentByValue();
	}

	public int getMinimum() {
		return minimum;
	}

	public void setMinimum(int minimum) {
		this.minimum = minimum;
		calculatePercentByValue();
	}

	private void calculatePercentByValue() {
		int valueDiff = maximum - minimum;
		setValueInPercent(100D * (value - minimum) / valueDiff);
	}

	public double getValueInPercent() {
		return valueInPercent;
	}

	public void setValueInPercent(double valueInPercent) {
		if (this.valueInPercent == valueInPercent) {
			return;
		}
		if (valueInPercent > 100D) {
			valueInPercent = 100D;
		} else if (valueInPercent < 0) {
			valueInPercent = 0D;
		}
		this.valueInPercent = valueInPercent;
		propagateCurrentValueToUI();
	}

	private void gatherValueInPercent() {
		if (layeredImage.isHorizontalMoveable()) {
			valueInPercent = getHorizontalValueInPercent();
		} else if (layeredImage.isVerticalMoveable()) {
			valueInPercent = getVerticalValueInPercent();
		}
		calculateValueByPercent();
	}

	private void calculateValueByPercent() {
		int valueDiff = maximum - minimum;
		value = (int) (valueDiff * valueInPercent / 100D) + minimum;
	}

	private void propagateCurrentValueToUI() {
		if (layeredImage.isHorizontalMoveable()) {
			setHorizontalValueInPercent(valueInPercent);
		} else if (layeredImage.isVerticalMoveable()) {
			setVerticalValueInPercent(valueInPercent);
		}
		refresh();
	}
}
