/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;

public class LabeledRotaryPotentiometerComponent extends Box {
	private static final long serialVersionUID = 1L;
	private final RotaryPotentiometerImageComponent potentiometer;
	private final JLabel jlabel;

	public LabeledRotaryPotentiometerComponent(String label) {
		this(label, null);
	}

	public LabeledRotaryPotentiometerComponent(String label, Float mid) {
		super(BoxLayout.Y_AXIS);
		if (mid != null) {
			potentiometer = RotaryPotentiometerImageComponent.createLabel3RotaryPotentiometerImageComponent();
			potentiometer.setMidValue(0F);
		} else {
			potentiometer = RotaryPotentiometerImageComponent.createLabel2RotaryPotentiometerImageComponent();
		}
		setOpaque(false);
		add(potentiometer);
		jlabel = new JLabel(label);
		jlabel.setAlignmentX(CENTER_ALIGNMENT);
		add(jlabel);
	}

	public void setValue(float value) {
		potentiometer.setValue(value);
		String format = potentiometer.formatPrecise(value);
		setToolTipText(format);
		potentiometer.setToolTipText(format);
		jlabel.setToolTipText(format);
	}
	
	public RotaryPotentiometerImageComponent getPotentiometer() {
		return potentiometer;
	}

	public JLabel getLabel() {
		return jlabel;
	}
}
