/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;

import javax.swing.JComponent;

import org.audiochain.ui.LightEmittingDiode;


/**
 * To define a LED, you simply have to provide the base color (i.e.
 * setDiodeBaseColor(0x00FF00) for a green LED or 0xFFFF00 for a yellow one or
 * use the defined Color constants of this class). To enable the LED set enabled
 * to true or false to disable it. After a change of color or enabled state,
 * call a repaint(). You can make a round LED if you decrease the radiusFactor
 * from its default value of 1.0 to i.e. 0.7.
 */
public class LightEmittingDiodeComponent extends JComponent {
	private static final long serialVersionUID = 1L;
	private final LightEmittingDiode diode;
	private String text;

	public LightEmittingDiodeComponent(int baseColor, int width, int height) {
		diode = new LightEmittingDiode(baseColor, width, height);
		init();
	}

	public LightEmittingDiodeComponent(int width, int height) {
		diode = new LightEmittingDiode(width, height);
		init();
	}
	
	private void init() {
		Dimension dim = new Dimension(diode.getWidth(), diode.getHeight());
		setPreferredSize(dim);
		setMinimumSize(dim);
		setMaximumSize(dim);
	}

	public LightEmittingDiodeComponent(LightEmittingDiode diode) {
		this.diode = diode;
	}

	@Override
	public void paint(Graphics g) {
		g.drawImage(diode.getBufferedImage(), 0, 0, null);
		if(text!=null) {
			BufferedImage stringImage = StringImageRenderer.drawNarrowString(text, getFont(), getForeground());
			g.drawImage(stringImage, getWidth()/2-stringImage.getWidth()/2, getHeight()/2-stringImage.getHeight()/2, null);
		}
	}

	public LightEmittingDiode getLightEmittingDiode() {
		return diode;
	}

	public boolean isLedEnabled() {
		return diode.isLedEnabled();
	}

	public void setLedEnabled(boolean enabled) {
		if (diode.isLedEnabled() != enabled) {
			diode.setLedEnabled(enabled);
			repaint();
		}
	}

	public String getText() {
		return text;
	}

	public void setText(String text) {
		this.text = text;
	}
}
