/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;

import javax.swing.JComponent;

import org.audiochain.ui.LightEmittingDiode;
import org.audiochain.ui.LightEmittingDiodeArray;
import org.audiochain.ui.LightEmittingDiodeMatrix;


public class LightEmittingDiodeMatrixComponent extends JComponent {
	private static final long serialVersionUID = 1L;
	private final LightEmittingDiodeMatrix lightEmittingDiodeMatrix;
	private final int arrayCount;
	private final int ledCountPerArray;
	private final int ledWidth;
	private final int ledHeight;
	private final int ledGap;
	private final int width;
	private final int height;

	public LightEmittingDiodeMatrixComponent(int arrayCount, int[] arrayBaseColors, int ledWidth, int ledHeight, int ledGap) {
		this.lightEmittingDiodeMatrix = new LightEmittingDiodeMatrix(arrayCount, arrayBaseColors, ledWidth, ledHeight);
		this.arrayCount = arrayCount;
		this.ledCountPerArray = arrayBaseColors.length;
		this.ledWidth = ledWidth;
		this.ledHeight = ledHeight;
		this.ledGap = ledGap;
		width = arrayCount * ledGap + 1 + arrayCount * ledWidth;
		height = ledCountPerArray * ledGap + 1 + ledCountPerArray * ledHeight;
		init();
	}

	private void init() {
		Dimension dim = new Dimension(width, height);
		setMinimumSize(dim);
		setPreferredSize(dim);
		setMaximumSize(dim);
	}

	@Override
	public void paint(Graphics g) {
		for (int x = ledGap, i = 0; x < width; x += ledGap + ledWidth, i++) {
			assert i < arrayCount;
			LightEmittingDiodeArray lightEmittingDiodeArray = lightEmittingDiodeMatrix.getLightEmittingDiodeArray(i);
			for (int y = ledGap, j = ledCountPerArray - 1; y < height; y += ledGap + ledHeight, j--) {
				LightEmittingDiode lightEmittingDiode = lightEmittingDiodeArray.getLightEmittingDiode(j);
				BufferedImage bufferedImage = lightEmittingDiode.getBufferedImage();
				g.drawImage(bufferedImage, x, y, null);
			}
		}
	}

	public LightEmittingDiodeMatrix getLightEmittingDiodeMatrix() {
		return lightEmittingDiodeMatrix;
	}
}
