/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui;

import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JLabel;

public class NotificationDialog {
	private static String HTML_START = "<html><body style='color:red'>";
	private static String HTML_END = "</body></html>";
	protected final Object caller;
	protected final JLabel notificationLabel;
	protected JButton applyButton;
	private final Map<Object, String> notificationMessageMap;

	public NotificationDialog(Object caller) {
		if (caller == null) {
			throw new IllegalArgumentException("Caller must not be null.");
		}
		this.caller = caller;
		this.notificationMessageMap = new LinkedHashMap<Object, String>();
		notificationLabel = new JLabel(" ");
	}

	protected final void putNotification(Object key, String newMessage) {
		if (newMessage == null) {
			notificationMessageMap.remove(key);
		} else {
			notificationMessageMap.put(key, newMessage);
		}

		StringBuilder sb = new StringBuilder();
		for (String message : notificationMessageMap.values()) {
			if (message != null) {
				sb.append(message);
				sb.append(' ');
			}
		}
		String msg = sb.toString().trim();
		if (msg.isEmpty()) {
			// prevent the label to be hidden
			notificationLabel.setText(" ");
			applyButton.setEnabled(mandatoryFieldsValid());
		} else {
			notificationLabel.setText(HTML_START + msg + HTML_END);
			applyButton.setEnabled(false);
		}
	}

	protected boolean hasNotification(Object key) {
		return notificationMessageMap.get(key) != null;
	}
	
	protected boolean mandatoryFieldsValid() {
		return true;
	}
}
