/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.sync;

import java.util.Collection;
import java.util.LinkedHashSet;

import org.audiochain.model.AudioChannels;
import org.audiochain.model.AudioProject;

class AudioProjectSyncObserver extends SyncAudioProjectModificationObserver {

	public AudioProjectSyncObserver(AudioProject project, CommandSyncSocket commandSyncSocket) {
		super(project, commandSyncSocket);
	}

	@Override
	public void audioProjectFrameLengthChanged() {
		
	}
	
	@Override
	public void audioProjectNameChanged(String oldValue, String newValue) {
		commandSyncSocket.printCommand(audioProjectNameChanged, getAudioProject().getIdentifier(), newValue);
	}
	
	private final SyncCommand audioProjectNameChanged = new SyncCommand("APNC") {
		@Override
		protected void execute(String... parameters) {
			AudioProject audioProject = commandSyncSocket.findAudioProjectByIdentifier(parameters[0]);
			if (audioProject == null) {
				return;
			}
			String newValue = parameters[1];
			audioProject.setName(newValue);
		}
	};

	@Override
	public void audioProjectSampleRateChanged(int oldValue, int newValue) {
		commandSyncSocket.printCommand(audioProjectSampleRateChanged, getAudioProject().getIdentifier(), String.valueOf(newValue));
	}
	
	private final SyncCommand audioProjectSampleRateChanged = new SyncCommand("APSRC") {
		@Override
		protected void execute(String... parameters) {
			AudioProject audioProject = commandSyncSocket.findAudioProjectByIdentifier(parameters[0]);
			if (audioProject == null) {
				return;
			}
			String newValue = parameters[1];
			audioProject.setSampleRate(Integer.parseInt(newValue));
		}
	};

	@Override
	public void audioProjectSampleSizeInBitsChanged(int oldValue, int newValue) {
		commandSyncSocket.printCommand(audioProjectSampleSizeInBitsChanged, getAudioProject().getIdentifier(), String.valueOf(newValue));
	}
	
	private final SyncCommand audioProjectSampleSizeInBitsChanged = new SyncCommand("APSSIBC") {
		@Override
		protected void execute(String... parameters) {
			AudioProject audioProject = commandSyncSocket.findAudioProjectByIdentifier(parameters[0]);
			if (audioProject == null) {
				return;
			}
			String newValue = parameters[1];
			audioProject.setSampleSizeInBits(Integer.parseInt(newValue));
		}
	};

	@Override
	public void audioProjectBigEndianChanged(boolean oldValue, boolean newValue) {
		commandSyncSocket.printCommand(audioProjectBigEndianChanged, getAudioProject().getIdentifier(), String.valueOf(newValue));
	}
	
	private final SyncCommand audioProjectBigEndianChanged = new SyncCommand("APBEC") {
		@Override
		protected void execute(String... parameters) {
			AudioProject audioProject = commandSyncSocket.findAudioProjectByIdentifier(parameters[0]);
			if (audioProject == null) {
				return;
			}
			String newValue = parameters[1];
			audioProject.setBigEndian(Boolean.parseBoolean(newValue));
		}
	};

	@Override
	public void audioProjectChannelsChanged(AudioChannels oldValue, AudioChannels newValue) {
		commandSyncSocket.printCommand(audioProjectChannelsChanged, getAudioProject().getIdentifier(), String.valueOf(newValue));
	}
	
	private final SyncCommand audioProjectChannelsChanged = new SyncCommand("APCC") {
		@Override
		protected void execute(String... parameters) {
			AudioProject audioProject = commandSyncSocket.findAudioProjectByIdentifier(parameters[0]);
			if (audioProject == null) {
				return;
			}
			String newValue = parameters[1];
			audioProject.setChannels(AudioChannels.valueOf(newValue));
		}
	};
	
	@Override
	public Collection<SyncCommand> getSyncCommands() {
		Collection<SyncCommand> syncCommands = new LinkedHashSet<SyncCommand>();
		syncCommands.add(audioProjectNameChanged);
		syncCommands.add(audioProjectSampleRateChanged);
		syncCommands.add(audioProjectSampleSizeInBitsChanged);
		syncCommands.add(audioProjectBigEndianChanged);
		syncCommands.add(audioProjectChannelsChanged);
		return syncCommands;
	}

}
