/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.sync;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

abstract class SyncCommand {
	private static final char NULL_REPLACEMENT = '\u0000';
	private static final char COMMAND_PARAMETER_SEPARATOR = '\u001E'; // record separator
	private final String name;

	protected SyncCommand(String name) {
		this.name = name;
	}

	protected boolean execIfMatches(String line) throws SyncSocketException {
		boolean matches = line.startsWith(name);
		if (matches) {
			List<String> parameters = new ArrayList<String>();
			String paramStr = line.substring(name.length() + 1);
			StringTokenizer st = new StringTokenizer(paramStr, "" + COMMAND_PARAMETER_SEPARATOR);
			while (st.hasMoreTokens()) {
				String parameter = st.nextToken();
				if (parameter.length() == 1 && parameter.charAt(0) == NULL_REPLACEMENT) {
					parameter = null;
				}
				parameters.add(parameter);
			}
			execute(parameters.toArray(new String[0]));
		}
		return matches;
	}

	abstract protected void execute(String... parameters) throws SyncSocketException;

	protected String getCommandString(String... parameters) {
		StringBuilder sb = new StringBuilder();
		sb.append(name);
		sb.append(':');
		for (String parameter : parameters) {
			sb.append(parameter != null ? parameter : NULL_REPLACEMENT);
			sb.append(COMMAND_PARAMETER_SEPARATOR);
		}
		return sb.toString();
	}

	protected String getName() {
		return name;
	}
}
